# Example for using pyanim library. Does a simulation of John Conway's
# game of life, and creates an animation over N frames of the simulation.

import pyanim
import numarray
from pylab import *

MAX_X = 100
MAX_Y = 100
PROB_CELL = 0.2
FRAMES = 100

# 1000k bitrate is very high, but gets rid of almost all distortions that
# can occur at lower bitrates (which is problematic for numbers on the graph)
BIT_RATE = 1000000

# how many frames per second
FRAME_RATE = 2

def get(pop, x, y):
  rx = x
  ry = y
  
  if x < 0: rx = MAX_X - 1
  elif x >= MAX_X: rx = 0

  if y < 0: ry = MAX_Y - 1
  elif y >= MAX_Y: ry = 0

  return pop[rx, ry]

def step(a, b):
  for x in range(0, MAX_X):
    for y in range(0, MAX_Y):
      count = sum([get(a, x - 1, y - 1),
                   get(a, x,     y - 1),
                   get(a, x + 1, y - 1),
                   get(a, x - 1, y    ),
                   get(a, x + 1, y    ),
                   get(a, x - 1, y + 1),
                   get(a, x,     y + 1),
                   get(a, x + 1, y + 1)])
      if count < 2:
        b[x, y] = 0
      elif count > 3:
        b[x, y] = 0
      elif count == 3:
        b[x, y] = 1

if __name__ == "__main__":
  a = numarray.zeros((MAX_X, MAX_Y))
  
  # create random initial population
  for x in range(0, MAX_X):
    for y in range(0, MAX_Y):
      if rand() < PROB_CELL:
        a[x, y] = 1

  # allow for interactive drawing
  ion()

  # draw population
  im = imshow(a, interpolation="nearest", vmin=0, vmax=1, cmap=cm.gray)

  ax = gca()
  manager = get_current_fig_manager()
  canvas = manager.canvas

  # these are used for what dimensions to make the animation
  width, height = canvas.get_width_height()

  # will select the correct video format based off of file extension
  video = pyanim.Video("test.mp4", width, height, FRAME_RATE, BIT_RATE)

  # draw the frames of the animation
  for frame in range(0, FRAMES):
    print "frame: ", frame

    # calculate next step of population
    b = a.copy()
    step(a, b)
    a = b.copy()
    im.set_data(a)
    draw()

    # this is a bit complicated rearrangement of data -- there should
    # be a more direct way to do this. Basically we're just grabbing
    # the data off of the canvas, and creating an RGB,RGB,RGB,... vector
    # from it
    buffer = canvas.tostring_rgb()
    bl = numarray.fromstring(buffer, numarray.UInt8)

    # write the RGB data as the next frame
    video.write_frame(bl.tolist())

  # finish writing the file
  video.close()
  
